﻿using GrapeCity.Forguncy.ServerApi;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace UserInfo
{
    public class UserInfoAPI: ForguncyApi
    {
        /// <summary>
        /// POST 発生時にユーザーアカウント管理に登録されているユーザー情報を取得して内部テーブルを刷新します。
        /// </summary>
        [Post]
        public void GetUsetInfo()
        {

            // 「登録ユーザー」テーブルのデータをすべて削除します。
            int retDel = DeleteTable("登録ユーザー", "登録ユーザー");

            // ユーザーアカウント管理に登録されているユーザー情報を取得します。
            List<GrapeCity.Forguncy.ServerApi.UserInfo> SrvUserInfos = new List<GrapeCity.Forguncy.ServerApi.UserInfo>();
            SrvUserInfos = (List<GrapeCity.Forguncy.ServerApi.UserInfo>)this.UserInfos.GetUserInfos();

            // 「登録ユーザー」テーブルへ登録するユーザー情報を生成します。
            var data = new Dictionary<string, ModifyData>();
            var modifyData = new ModifyData();

            foreach (var SrvUserInfo in SrvUserInfos)
            {
                // Administratorは除外します。
                if(!SrvUserInfo.UserName.Equals("Administrator"))
                {
                    // 1レコードの保存データを生成します。
                    modifyData.AddRows.Add(new Dictionary<string, object>
                        {
                            {"ユーザー名", SrvUserInfo.UserName },
                            {"氏名", SrvUserInfo.FullName},
                            {"メールアドレス", SrvUserInfo.Email},
                            {"ロール", SrvUserInfo.Role }
                        });
                }
            }

            // データベースへ書き込みます。
            data.Add("登録ユーザー", modifyData);
            this.DataAccess.ModifyTablesData(data);
        }

        /// <summary>
        /// 指定したテーブルのレコードを削除します。ODataで削除の条件を指定できます。
        /// </summary>
        /// <param name="strTableName">テーブル名</param>
        /// <param name="strODataCondition">削除するレコードのODataでの抽出条件。テーブル名を指定すると全データが対象になります。</param>
        /// <returns></returns>
        private int DeleteTable(string strTableName, string strODataCondition)
        {
            int intReturnl = 0;

            // GetTableData (string odataPath)メソッドのOData指定を利用して各レコードの情報を取得します。
            List<Dictionary<string, object>> dicRecords = new List<Dictionary<string, object>>();
            dicRecords = (List<Dictionary<string, object>>)this.DataAccess.GetTableData(strODataCondition);

            // 削除情報を生成します。
            var data = new Dictionary<string, ModifyData>();
            var modifyData = new ModifyData();

            foreach (Dictionary<string, object> dicRecord in dicRecords)
            {
                // フィールド情報を取り出します。
                Dictionary<string, object> dicFields = new Dictionary<string, object>();
                dicFields = (Dictionary<string, object>)dicRecord;

                // 削除情報を生成します。
                modifyData.DeleteRows.Add(new ColumnValuePair()
                {
                    ColumnName = "ID",
                    Value = dicRecord["ID"]
                });
                intReturnl += 1;
            }

            // テーブルの変更を反映します。
            data.Add(strTableName, modifyData);
            this.DataAccess.ModifyTablesData(data);

            return (intReturnl);

        }

        // 毎日午前0時にメソッドが実行されます。
        [ScheduleApi("0:00:0", "1")]
        public void ScheduleGetUsetInfo()
        {
            // ユーザーアカウント管理に登録されているユーザー情報を取得して
            // 「登録ユーザー」テーブルを刷新します。
            GetUsetInfo();
        }

    }
}
